@@ -1,22 +1,4 @@
 <?php
-/*
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- * This software consists of voluntary contributions made by many individuals
- * and is licensed under the MIT license. For more information, see
- * <http://www.doctrine-project.org>.
- */
-
 namespace Doctrine\Bundle\DoctrineCacheBundle\DependencyInjection;
 
 use Symfony\Component\Config\Definition\Builder\TreeBuilder;
@@ -74,7 +56,6 @@ class Configuration implements ConfigurationInterface
     public function getProviderNames(NodeInterface $tree)
     {
         foreach ($tree->getChildren() as $providers) {
-
             if ($providers->getName() !== 'providers') {
                 continue;
             }
@@ -105,8 +86,8 @@ class Configuration implements ConfigurationInterface
     public function getConfigTreeBuilder()
     {
         $self            = $this;
-        $builder         = new TreeBuilder();
-        $node            = $builder->root('doctrine_cache', 'array');
+        $builder         = new TreeBuilder('doctrine_cache');
+        $node            = $this->getRootNode($builder, 'doctrine_cache');
         $normalization   = function ($conf) use ($self, $builder) {
             $conf['type'] = isset($conf['type'])
                 ? $conf['type']
@@ -117,6 +98,7 @@ class Configuration implements ConfigurationInterface
                 $options = reset($params);
                 $conf    = array(
                     'type'            => 'custom_provider',
+                    'namespace' => isset($conf['namespace']) ? $conf['namespace'] : null ,
                     'custom_provider' => array(
                         'type'      => $conf['type'],
                         'options'   => $options ?: null,
@@ -216,8 +198,8 @@ class Configuration implements ConfigurationInterface
      */
     private function addBasicProviderNode($name)
     {
-        $builder = new TreeBuilder();
-        $node    = $builder->root($name);
+        $builder = new TreeBuilder($name);
+        $node    = $this->getRootNode($builder, $name);
 
         return $node;
     }
@@ -229,8 +211,8 @@ class Configuration implements ConfigurationInterface
      */
     private function addCustomProviderNode()
     {
-        $builder = new TreeBuilder();
-        $node    = $builder->root('custom_provider');
+        $builder = new TreeBuilder('custom_provider');
+        $node    = $this->getRootNode($builder, 'custom_provider');
 
         $node
             ->children()
@@ -252,8 +234,8 @@ class Configuration implements ConfigurationInterface
      */
     private function addChainNode()
     {
-        $builder = new TreeBuilder();
-        $node    = $builder->root('chain');
+        $builder = new TreeBuilder('chain');
+        $node    = $this->getRootNode($builder, 'chain');
 
         $node
             ->fixXmlConfig('provider')
@@ -274,8 +256,8 @@ class Configuration implements ConfigurationInterface
      */
     private function addMemcacheNode()
     {
-        $builder = new TreeBuilder();
-        $node    = $builder->root('memcache');
+        $builder = new TreeBuilder('memcache');
+        $node    = $this->getRootNode($builder, 'memcache');
         $host    = '%doctrine_cache.memcache.host%';
         $port    = '%doctrine_cache.memcache.port%';
 
@@ -319,8 +301,8 @@ class Configuration implements ConfigurationInterface
      */
     private function addMemcachedNode()
     {
-        $builder = new TreeBuilder();
-        $node    = $builder->root('memcached');
+        $builder = new TreeBuilder('memcached');
+        $node    = $this->getRootNode($builder, 'memcached');
         $host    = '%doctrine_cache.memcached.host%';
         $port    = '%doctrine_cache.memcached.port%';
 
@@ -365,8 +347,8 @@ class Configuration implements ConfigurationInterface
      */
     private function addRedisNode()
     {
-        $builder = new TreeBuilder();
-        $node    = $builder->root('redis');
+        $builder = new TreeBuilder('redis');
+        $node    = $this->getRootNode($builder, 'redis');
 
         $node
             ->addDefaultsIfNotSet()
@@ -377,6 +359,7 @@ class Configuration implements ConfigurationInterface
                 ->scalarNode('password')->defaultNull()->end()
                 ->scalarNode('timeout')->defaultNull()->end()
                 ->scalarNode('database')->defaultNull()->end()
+                ->booleanNode('persistent')->defaultFalse()->end()
             ->end()
         ;
 
@@ -390,8 +373,8 @@ class Configuration implements ConfigurationInterface
      */
     private function addPredisNode()
     {
-        $builder = new TreeBuilder();
-        $node    = $builder->root('predis');
+        $builder = new TreeBuilder('predis');
+        $node    = $this->getRootNode($builder, 'predis');
 
         $node
             ->addDefaultsIfNotSet()
@@ -420,8 +403,8 @@ class Configuration implements ConfigurationInterface
      */
     private function addRiakNode()
     {
-        $builder = new TreeBuilder();
-        $node    = $builder->root('riak');
+        $builder = new TreeBuilder('riak');
+        $node    = $this->getRootNode($builder, 'riak');
 
         $node
             ->addDefaultsIfNotSet()
@@ -450,8 +433,8 @@ class Configuration implements ConfigurationInterface
      */
     private function addCouchbaseNode()
     {
-        $builder = new TreeBuilder();
-        $node    = $builder->root('couchbase');
+        $builder = new TreeBuilder('couchbase');
+        $node    = $this->getRootNode($builder, 'couchbase');
 
         $node
             ->addDefaultsIfNotSet()
@@ -478,8 +461,8 @@ class Configuration implements ConfigurationInterface
      */
     private function addMongoNode()
     {
-        $builder = new TreeBuilder();
-        $node    = $builder->root('mongodb');
+        $builder = new TreeBuilder('mongodb');
+        $node    = $this->getRootNode($builder, 'mongodb');
 
         $node
             ->addDefaultsIfNotSet()
@@ -502,8 +485,8 @@ class Configuration implements ConfigurationInterface
      */
     private function addPhpFileNode()
     {
-        $builder = new TreeBuilder();
-        $node    = $builder->root('php_file');
+        $builder = new TreeBuilder('php_file');
+        $node    = $this->getRootNode($builder, 'php_file');
 
         $node
             ->addDefaultsIfNotSet()
@@ -524,8 +507,8 @@ class Configuration implements ConfigurationInterface
      */
     private function addFileSystemNode()
     {
-        $builder = new TreeBuilder();
-        $node    = $builder->root('file_system');
+        $builder = new TreeBuilder('file_system');
+        $node    = $this->getRootNode($builder, 'file_system');
 
         $node
             ->addDefaultsIfNotSet()
@@ -546,8 +529,8 @@ class Configuration implements ConfigurationInterface
      */
     private function addSqlite3Node()
     {
-        $builder = new TreeBuilder();
-        $node    = $builder->root('sqlite3');
+        $builder = new TreeBuilder('sqlite3');
+        $node    = $this->getRootNode($builder, 'sqlite3');
 
         $node
             ->addDefaultsIfNotSet()
@@ -560,4 +543,14 @@ class Configuration implements ConfigurationInterface
 
         return $node;
     }
+
+    private function getRootNode(TreeBuilder $treeBuilder, $name)
+    {
+        // BC layer for symfony/config 4.1 and older
+        if ( ! \method_exists($treeBuilder, 'getRootNode')) {
+            return $treeBuilder->root($name);
+        }
+
+        return $treeBuilder->getRootNode();
+    }
 }
